/*
 * (c) 3M Company 2012. Generated in 2012 as an unpublished copyrighted work.
 * This program contains confidential proprietary information of 3M. Such
 * information may not be used or reproduced without prior written consent
 * of 3M.
 *
 * Program:  testapr.c - test limited APR for AHRQ
 */

#include <stdio.h>
#include <string.h>
#include <windows.h>
#include "aprlim.h"

enum {ARG_PROG, ARG_DLL, ARG_APRROT, ARG_MAPROT, ARG_INFILE, ARG_LIMIT, ARG_COUNT};

void        loadFields(char *buffer);
int         getInteger(int fld);
char        *getString(int fld);
static char buffer[2048];

enum { /* UTF field numbers */
	FLD_CaseID  =  0,
	FLD_Version =  2,
	FLD_NumDx   =  5,
	FLD_DxWidth =  6,
	FLD_Dx      =  7,
	FLD_NumPr   =  9,
	FLD_PrWidth = 10,
	FLD_Procs   = 11,
	FLD_AgeYrs  = 12,
	FLD_Sex     = 13,
	FLD_Dstat   = 14,
	FLD_AgeDA   = 15,
	FLD_AgeDD   = 16,
	FLD_BWO     = 17,
	FLD_BWGT    = 18,
	FLD_DMV     = 19,
	FLD_CodeYr  = 20,
	FLD_CodeQtr = 21,
	FLD_GRC     = 25,
	FLD_MDC     = 26,
	FLD_DRG     = 27,
	FLD_SOI     = 28,
	FLD_ROM     = 29,
	FLD_EDRG    = 30
};


int main(int argc, char **argv)
{
	AprLimOpenFcn  AprLimOpen;
	AprLimCloseFcn AprLimClose;
	AprLimGroupFcn AprLimGroup;
	AprHANDLE      apr;
	HMODULE        hDLL;

	int  rc;
	int  count = 0;
	int  mismatch = 0;
	int  bad;
	int  version, numdx, dxwidth, numpr, prwidth, ageyrs;
	int  sex, dstat, ageda, agedd, bwo, bwgt, dmv;
	int  codeyr, codeqtr;
	int  grc, drg, soi, rom;
	int  egrc, edrg, esoi, erom;
	int  limit;
	char *caseid;
	char *diagnoses;
	char *procedures;
	char *p;
	FILE *infile;

	if (argc < ARG_COUNT) {
		fputs("usage: testapr DLL aprrot maprot input_file limit\n", stderr);
		return 0;
	}

	limit = atoi(argv[ARG_LIMIT]);

	p    = argv[ARG_DLL];
	hDLL = LoadLibraryA(p);
	if (!hDLL) {
		fprintf(stderr, "Cannot open library %s (%d)\n", p, GetLastError());
		return 1;
	}

	AprLimOpen = (AprLimOpenFcn)GetProcAddress(hDLL, "AprLimOpen");
	if (!AprLimOpen) {
		FreeLibrary(hDLL);
		fprintf(stderr, "Cannot find AprLimOpen (%d)\n", GetLastError());
		return 1;
	}

	AprLimClose = (AprLimCloseFcn)GetProcAddress(hDLL, "AprLimClose");
	if (!AprLimClose) {
		FreeLibrary(hDLL);
		fprintf(stderr, "Cannot find AprLimClose (%d)\n", GetLastError());
		return 1;
	}

	AprLimGroup = (AprLimGroupFcn)GetProcAddress(hDLL, "AprLimGroup");
	if (!AprLimGroup) {
		FreeLibrary(hDLL);
		fprintf(stderr, "Cannot find AprLimGroup (%d)\n", GetLastError());
		return 1;
	}

	rc = AprLimOpen(argv[ARG_APRROT], argv[ARG_MAPROT], &apr);
	if (rc) {
		fprintf(stderr, "Error %d opening ROTs APR=%s MAP=%s\n", rc, argv[ARG_APRROT], argv[ARG_MAPROT]);
		return 1;
	}

	p = argv[ARG_INFILE];

	if (fopen_s(&infile, p, "r" ) != 0) {
		fprintf(stderr, "Cannot open %s\n", p);
		return 1;
	}

	while (fgets(buffer, sizeof(buffer), infile)) {
		if (limit && count >= limit) break;

		for (p = buffer + strlen(buffer) - 1; p >= buffer; p -= 1) {
			if (*p > ' ') break;
			*p = 0;
		}

		count += 1;

		#ifndef NO_BLIP
		if ((count % 1000) == 0)
			fprintf(stderr, "%7d\r", count);
		#endif

		loadFields(buffer);

		caseid     = getString(FLD_CaseID);

		version    = getInteger(FLD_Version);
		numdx      = getInteger(FLD_NumDx);
		dxwidth    = getInteger(FLD_DxWidth);

		diagnoses  = getString(FLD_Dx);

		numpr      = getInteger(FLD_NumPr);
		prwidth    = getInteger(FLD_PrWidth);

		procedures = getString(FLD_Procs);

		ageyrs     = getInteger(FLD_AgeYrs);
		sex        = getInteger(FLD_Sex);
		dstat      = getInteger(FLD_Dstat);
		ageda      = getInteger(FLD_AgeDA);
		agedd      = getInteger(FLD_AgeDD);
		bwo        = getInteger(FLD_BWO);
		bwgt       = getInteger(FLD_BWGT);
		dmv        = getInteger(FLD_DMV);
		codeyr     = getInteger(FLD_CodeYr);
		codeqtr    = getInteger(FLD_CodeQtr);
		egrc       = getInteger(FLD_GRC);
		edrg       = getInteger(FLD_DRG);
		esoi       = getInteger(FLD_SOI);
		erom       = getInteger(FLD_ROM);

		grc = AprLimGroup(apr,codeyr,codeqtr,version,numdx,dxwidth,diagnoses,numpr,prwidth,procedures,ageyrs,ageda,agedd,sex,dstat,bwo,bwgt,dmv,&drg,&soi,&rom);

		bad = 0;

		if (egrc != grc) {
			printf("%s (%d/%d) GRC: %d expected, %d actual\n", caseid, codeyr, codeqtr, egrc, grc);
			bad = 1;
		}

		if (edrg != drg) {
			printf("%s (%d/%d) DRG: %d expected, %d actual\n", caseid, codeyr, codeqtr, edrg, drg);
			bad = 1;
		}

		if (esoi != soi) {
			printf("%s (%d/%d) SOI: %d expected, %d actual\n", caseid, codeyr, codeqtr, esoi, soi);
			bad = 1;
		}

		if (erom != rom) {
			printf("%s (%d/%d) ROM: %d expected, %d actual\n", caseid, codeyr, codeqtr, erom, rom);
			bad = 1;
		}

		if (bad)
			mismatch += 1;
	}

	fprintf(stderr, "%7d cases processed\n", count);
	fprintf(stderr, "%7d mismatches\n",      mismatch);

	fclose(infile);
	AprLimClose(apr);
	FreeLibrary(hDLL);

	return 0;
}
